#include <windows.h>
#include "IniFile.h"
#include "w32_exptsppt.h"
#include "AddUnicode.h"

// default values: (see IniFile.h for explanation!)

#define TEXT_FILENAME TEXT("reading_script.txt")
#define FONT_NAME TEXT("Times Roman")
#define FONT_HEIGHT SCRHEIGHT/25 
#define LINE_SPACING SCRHEIGHT/14
#define TIME_LIMIT 20000L
#define CURSOR_SIZE 4
// *** 11.6.2001 - negative value enables additional debug mode
#define PADDING_SIZE 6
// *** following are 2.7.2001
// *** negative means only leave N pixels between lines
#define NO_DC_DELAY 500
#define WAVE_FREQ 16000
// *** also fixed to mono 16bit, should better be parsed from RIFF WAVE header!
#define SAVE_TEXT_BITMAPS 0
// *** if 1 (not 0), bitmaps made by w32_text_bitmap will be stored

///////////////////////////////////////
#define MAX_STRING 1024
LPCTSTR gpszDefaultIniName = TEXT("reading.ini");
LPCTSTR gpszIniSection = TEXT("TextParams");
TCHAR gpszIniFilename[MAX_STRING];

LPCTSTR gpszEntry_TextFilename = TEXT("text_filename");
LPCTSTR gpszEntry_FontName = TEXT("font_name");
LPCTSTR gpszEntry_FontHeight = TEXT("font_height");
LPCTSTR gpszEntry_LineSpacing = TEXT("line_spacing");
LPCTSTR gpszEntry_TimeLimit = TEXT("time_limit");
LPCTSTR gpszEntry_CursorSize = TEXT("cursor_size");
// *** next are 2.7.2001
LPCTSTR gpszEntry_TextPaddingSize = TEXT("text_padding_size");
LPCTSTR gpszEntry_NoDCDelay = TEXT("no_dc_delay");
LPCTSTR gpszEntry_WaveFreq = TEXT("wave_freq");
LPCTSTR gpszEntry_SaveTextBitmaps = TEXT("save_text_bitmaps");

TEXTPARAMS gIniPara;  // holds parameters of ini file

////////////////////////////////////////////
void GetIniName()
{
	DWORD dwRet;
	DWORD dwRet2;
	TCHAR gpszWinDir[MAX_STRING];

	dwRet = GetCurrentDirectory(MAX_STRING, gpszIniFilename);

	// *** 2.7.2001 complain if current == windows
	dwRet2 = GetWindowsDirectory(gpszWinDir, MAX_STRING);
	if (dwRet > 0 && dwRet < MAX_STRING &&
		dwRet2 > 0 && dwRet2 < MAX_STRING &&
		(lstrcmp(gpszWinDir, gpszIniFilename) == 0))
	{
		MessageBox(NULL, 
			TEXT("Current directory is WINDOWS directory, all files will end up there!"),
			TEXT("Error in GetIniName and more"), MB_ICONWARNING);
	} 

	if (dwRet > 0 && dwRet < MAX_STRING)
	{
		lstrcat(gpszIniFilename, TEXT("\\"));
		lstrcat(gpszIniFilename, gpszDefaultIniName);
	}
	else
	{
		MessageBox(NULL, 
			TEXT("Could not get current directory. INI-file created in WINDOWS directory."), 
			TEXT("Error in GetIniName"), MB_ICONWARNING);

		lstrcpy(gpszIniFilename, gpszDefaultIniName);
	}
}

BOOL ReadInt(PLONG pl, LPCTSTR pszEntry)
{
	const UINT nErrVal = 0xFFFF;
	UINT nVal = GetPrivateProfileInt(gpszIniSection, (LPCTSTR) pszEntry, nErrVal, gpszIniFilename);
	if (nVal == nErrVal)
	{
		*pl = 0;
		return FALSE;
	}
	else 
	{
		*pl = (long) nVal;
		return TRUE;
	}
}

BOOL ReadString(TCHAR* pszString, LPCTSTR pszEntry)
{
	TCHAR pcBuf[CHAR_MAX];
	DWORD dwRet;
	dwRet = GetPrivateProfileString(gpszIniSection, (LPCTSTR) pszEntry, TEXT(""), pcBuf, CHAR_MAX, gpszIniFilename);
	lstrcpy(pszString, pcBuf);

	if (lstrlen(pszString) == 0)
		return FALSE;
	else
		return TRUE;
}

BOOL WriteString(TCHAR* pszString, LPCTSTR pszEntry)
{
	BOOL bRet = WritePrivateProfileString(gpszIniSection, (LPCTSTR) pszEntry, pszString, gpszIniFilename);
	return bRet;
}
	
BOOL WriteInt(long n, LPCTSTR pszEntry)
{
	BOOL bRet;
	TCHAR pcBuf[DIGIT_MAX];
	wsprintf(pcBuf, TEXT("%d"), n);
	bRet = WritePrivateProfileString(gpszIniSection, (LPCTSTR) pszEntry, pcBuf, gpszIniFilename);
	return bRet;
}	

BOOL ReadParams(PTEXTPARAMS pPara)
{
	BOOL bRet;

	GetIniName();

	bRet = ReadString(pPara->pszTextFilename, gpszEntry_TextFilename);
	bRet &= ReadString(pPara->pszFontName, gpszEntry_FontName);	
	bRet &= ReadInt(&pPara->lFontHeight, gpszEntry_FontHeight);
	bRet &= ReadInt(&pPara->lLineSpacing, gpszEntry_LineSpacing);
	bRet &= ReadInt(&pPara->lTimeLimit, gpszEntry_TimeLimit);
	bRet &= ReadInt(&pPara->lCursorSize, gpszEntry_CursorSize); // 11.6.2001
	bRet &= ReadInt(&pPara->lTextPaddingSize, gpszEntry_TextPaddingSize); // 2.7.2001
	bRet &= ReadInt(&pPara->lNoDCDelay, gpszEntry_NoDCDelay); // 2.7.2001
	bRet &= ReadInt(&pPara->lWaveFreq, gpszEntry_WaveFreq); // 2.7.2001
	bRet &= ReadInt(&pPara->lSaveTextBitmaps, gpszEntry_SaveTextBitmaps); // 2.7.2001

	return bRet;
}

BOOL WriteParams(PTEXTPARAMS pPara)
{
	BOOL bRet;

	GetIniName();

	bRet = WriteString(pPara->pszTextFilename,gpszEntry_TextFilename);
	bRet &= WriteString(pPara->pszFontName, gpszEntry_FontName);	
	bRet &= WriteInt(pPara->lFontHeight,gpszEntry_FontHeight);
	bRet &= WriteInt(pPara->lLineSpacing,gpszEntry_LineSpacing);
	bRet &= WriteInt(pPara->lTimeLimit,gpszEntry_TimeLimit);
	bRet &= WriteInt(pPara->lCursorSize,gpszEntry_CursorSize); // 11.6.2001
	bRet &= WriteInt(pPara->lTextPaddingSize, gpszEntry_TextPaddingSize); // 2.7.2001
	bRet &= WriteInt(pPara->lNoDCDelay, gpszEntry_NoDCDelay); // 2.7.2001
	bRet &= WriteInt(pPara->lWaveFreq, gpszEntry_WaveFreq); // 2.7.2001
	bRet &= WriteInt(pPara->lSaveTextBitmaps, gpszEntry_SaveTextBitmaps); // 2.7.2001

	return bRet;
}

void GetDefaultValues(PTEXTPARAMS pPara)
{
	lstrcpy(pPara->pszTextFilename, TEXT_FILENAME);
	lstrcpy(pPara->pszFontName, FONT_NAME);	
	pPara->lFontHeight = FONT_HEIGHT;
	pPara->lLineSpacing = LINE_SPACING;
	pPara->lTimeLimit = TIME_LIMIT;
	pPara->lCursorSize = CURSOR_SIZE; // 11.6.2001
	// *** next are 2.7.2001
	pPara->lTextPaddingSize = PADDING_SIZE;
	pPara->lNoDCDelay = NO_DC_DELAY;
	pPara->lWaveFreq = WAVE_FREQ;
	pPara->lSaveTextBitmaps = SAVE_TEXT_BITMAPS;
}
