/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* This file: Gaze-controlled selection, including on-line drift correction */

// #define LOG_FIXUPDATE 42	/* to log all fixupdate events */
#define FIXUPDATE_MSEC 4	/* timing resolution */
// #define DEBUG_AREAS_DELAY 2500
		/* time to show the graphical areas in debug mode */
#define DEBUG_FLASHES 50	/* how often the image is toggled between */
	/* areas and stimulus to reveal areas for debugging at the beginning */

/* *** 3/2001 - was w32_gcontrol_trial way before (and w32_bitmap_trial) *** */
/* *** Modified by Eric Auer: 1000 regions, no longer highlights regions *** */
/* *** but instead does only logging. Gaze at first region activates     *** */
/* *** stimulus presentation (first region is marked by gaze target      *** */
/* *** before) trial ends if valid key is pressed or timeout occurs      *** */


/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

#include <windows.h>
#include <windowsx.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"	/* header file for this experiment */

#include "global_regions.h"     /* *** has rgn[] *** */
#include "reading_loadscript.h" /* *** trial struct! *** */
#include "IniFile.h"	/* *** 11.6.2001 - want to use global config here... *** */

#include "string.h"		/* snprintf or sprintf or whatever */

#include "ea_sound.h"	/* *** 11.6.2001 win95 sound support by Eric *** */

int last_region = -1;	  // region with gaze at last update
int current_region = -1;  // region currently accumulating gaze time

#define TRIGGER_MAIN_THRESHOLD 500	/* msec of gaze to trigger main stimulus */
					/* Warning: also needed for implicit drift correction! */
// long dwell_threshold = 700;	// msec of gaze in region required to trigger

int used_regions = 0; 		/* *** */
UINT32 trial_start = -1;

PTRIALPARAMS myTrialGlobal; // *** 11.6.2001 for which_region stuff

/*********** DISPLAY GRAPHICS ************/

// see w32_bitmap_trial.c for draw_grid_to_bitmap() and display_bitmap()

              // inverts RECT area to highlight/unhighlight selection
void invert_rect(RECT *r) 
{                      
  HDC hdc;         // display drawing context

  if(!full_screen_window) return;    // make sure window is still open 
  hdc = GetDC(full_screen_window);   // get drawing context 

  SetROP2(hdc, R2_NOT);                                  // operation = invert
  Rectangle(hdc, r->left, r->top, r->right, r->bottom);  // invert rectangle (don't need brush for invert)
  SetROP2(hdc, R2_COPYPEN);                              // reset GDI mode
  ReleaseDC(full_screen_window, hdc);                    // release context
}


/********* GAZE CONTROL REGIONS *********/

	// This code must determine:
	// - which region an update is in
	// - update region dwell times
	// - trigger on dwell time
	// - do fixup for drift

		// process triggering of a new region:
                // remove highlight from old region, highlight new region
		// *** removed highlighting stuff!
void trigger_region(int region)
{
  int i;
  
  if ((region<0) || (region>=used_regions)) return;
  for(i=0; i<used_regions; i++)   // scan thru all regions
    { rgn[i].triggered = 0;
		// here we could highlight and un-highlight stuff
    }
  rgn[region].triggered = 1;
} 			


           // Clear region data when gaze switches to new region	
void reset_regions(void)
{
  int i;
  
  for(i=0; i<used_regions; i++)   // reset data for all regions
    {
      rgn[i].avgxsum = 0;   
      rgn[i].avgysum = 0;
      rgn[i].dwell = 0;
    }
  current_region = -1;      // no region currently selected
  last_region = -1;
} 			


// *** new 11.6.2001
int color2region(COLORREF myColor)
{
	// a COLORREF is an unsigned long...
	unsigned long i2;
	int i;
	i = 0;
	i2 = (myColor & 255);
	i2 = (i2<1) ? 0 : ((i2-1)>>6);
	i += i2;
	i2 = ((myColor>>8) & 255);
	i2 = (i2<1) ? 0 : ((i2-1)>>6);
	i += (i2<<2);
	i2 = ((myColor>>16) & 255);
	i2 = (i2<1) ? 0 : ((i2-1)>>6);
	i += (i2<<4);
	i2 = ((myColor>>24) & 255);
	i2 = (i2<1) ? 0 : ((i2-1)>>6);
	i += (i2<<6);
	return i;
}

	// Determine which region gaze is in
	// return 0..used_regions-1 for a valid region, -1 if not in any region
int which_region(int x, int y)
{
	int i;
	POINT pt;
	// *** 11.6.2001 - guess this can be done
	// *** much easier...
/**/	HDC memDC;
/**/	HDC myDC;
/**/	HBITMAP oldBitMap;
	BITMAP myBitMap;

	pt.x = x;                    // set POINT with gaze location
	pt.y = y;
	if (myTrialGlobal->bHasImage == TRUE)
	{ // *** new 11.6.2001: allow graphical region definition
/**/	myDC = GetDC(full_screen_window);
/**/	memDC = CreateCompatibleDC(myDC);
		// *** compat to window? compat to bmp
		// *** and pal based would be better!
/**/	oldBitMap = SelectObject(memDC, myTrialGlobal->hTplBmp);
		GetObject(myTrialGlobal->hTplBmp,
			sizeof(BITMAP), &myBitMap);
		if (x<0 || y<0 ||
			x>=myBitMap.bmWidth ||
			y>=myBitMap.bmHeight)
		{
			i = -1; // x or y out of range
		}
		else
		{
/**/		i = color2region(GetPixel(memDC,x,y));
			i = i % (NREGIONS-1);
				// *** evil hack, better
				// *** stop using rgn[]
				// *** completely if has image!!!
		}
		// *** this was evil...
		// *** ... but this is worse (broken!):
		// *** i = (myBitMap.bmBitsPixel * x) + (myBitMap.bmWidthBytes * y);
		// *** i = ((int *)myBitMap.bmBits)[i];

/**/	SelectObject(memDC,oldBitMap); // restore
/**/	DeleteDC(memDC); // release
/**/	ReleaseDC(full_screen_window,myDC); // release
		return i;
		// or get HDC by GetDC from HWND, then GetPixel(HDC,x,y);
	}
	else
	{
		for(i=0; i<used_regions; i++)      // scan all regions for match
			if(PtInRect(&(rgn[i].rsense), pt))
			{
				return i;
			}
		return -1;                   // not in any region
	}
}

		// *** NEW function to present the stimulus !
		// *** if log_synctime is 1, the log message
		// *** that marks zero time for EDFVIEW will be written
UINT32 present_display(int log_synctime, HBITMAP myBitmap) // returns the start time
{
    // DISPLAY OUR IMAGE TO SUBJECT  by copying bitmap to display
    // By waiting for the display refresh before and after drawing, we will
    // have an accurate idea of the time the subject saw the stimulus.

  wait_for_video_refresh();	// synchronize before drawing so all seen at once
  display_bitmap(full_screen_window, myBitmap, 0, 0);  // show BITMAP
  wait_for_video_refresh();  	 // synchronize to refresh
  eyemsg_printf("DISPLAY ON");	 // message for RT recording in analysis 
  if (log_synctime != 0)
  {
	trial_start = current_time();  // record the display onset time 
    eyemsg_printf("SYNCTIME");	 // message marks zero-plot time for EDFVIEW 
  }
  DeleteObject(myBitmap);        // Delete the bitmap 

  if (myTrialGlobal->bHasSound)
  {
	msec_delay(50); // wait SHORT while (*** hint: make configable ***)
  	eyemsg_printf("STARTING SOUND");
		// *** 18.6.2001 timing check 1
	eaSoundPlay();  // *** 11.6.2001 - start playing that sound
  	eyemsg_printf("SOUND STARTED %ld",eaSoundWhere());
		// *** 18.6.2001 timing check 2, first eaSoundWhere call
		// (careful if eaSoundPlay starts a delay
		// in the background, "where" may be -1 !
	// eaSoundPlay should return at once, if further
	// loading is needed, it should be done in the
	// background!
  }

  return trial_start;
}

		// Process a fixation-update event:
		// Detect and handle a switch between regions
		// Otherwise, accumulate time and position in region
		// Trigger region when time exceeds threshold
// *** reset vars if stimulus_on < 1, gbm is stimulus
int process_fixupdate(int x, int y, long dur,
		HBITMAP gbm, int stimulus_on)
		// returns new state of stimulus_on
{
static int avgx, avgy;
   RECT livecursor;			// *** show real time cursor for debugging
static int livex,livey;		// *** last position of cursor
   int i,j;
   long playtime;	// *** 11.6.2001 - current sound playing position
static int ishown=0; // for the debug display
static int ptshown=0; // also for the debug display

   i = which_region(x,y);	// find out which region is gaze in

   if (stimulus_on < 0)		// < 0 is uninitialized, 0 is false, 1 is true
   {
	   avgx = 0;
	   avgy = 0;
	   livex = x;	// (init here not really needed, see below)
	   livey = y;

	   reset_regions();	// clear averages and dwell time
	   stimulus_on = 0;
   }
   else
   {
		if (gIniPara.lCursorSize!=0)
		{ // *** DEBUG MODE: GAZE CURSOR
			j = gIniPara.lCursorSize;
			j = (j<0) ? -j : j;
			SetRect(&livecursor, livex-j,
				livey-j, livex+j, livey+j);
			invert_rect(&livecursor);	// *** hide real time cursor at old position
		}
	}
		if (gIniPara.lCursorSize!=0)
		{ // *** DEBUG MODE: GAZE CURSOR
			j = gIniPara.lCursorSize;
			j = (j<0) ? -j : j;
			SetRect(&livecursor, x-j, y-j, x+j, y+j);
			invert_rect(&livecursor);	// *** show real time cursor for debugging
			livex = x;
			livey = y;
		}
	/* *** *** We do the first part only if stimulus is still invisible *** *** */
	// *** ... which should never happen for graphical trials, but who knows

	if ((stimulus_on < 1)
		&& (myTrialGlobal->bHasImage == FALSE))
		// *** only wait for activating stimulus
	{
		if (i == -1)	// not any region...
		{
			if (last_region == -1)	// ... for two times in a row
			{
				rgn[0].dwell = 0;	// reset dwell time and average
				rgn[0].avgxsum = 0;
				rgn[0].avgysum = 0;
			}
		}
		if (i == 0)		// sensitive region
		{
			rgn[0].dwell += dur;	// add time and build average
			rgn[0].avgxsum += dur * x;
			rgn[0].avgysum += dur * y;
		}
		if ((i == 0) && (last_region == 0)	/* kind of stable */
			&& (rgn[0].dwell >= TRIGGER_MAIN_THRESHOLD))
		{
			// NOW we activate the stimulus
			stimulus_on = 1;
           trigger_region(i);		        // TRIGGERED:
           avgx = rgn[i].avgxsum / rgn[i].dwell;    // compute avg. gaze position
           avgy = rgn[i].avgysum / rgn[i].dwell;
            // correct for drift (may cause false saccade in data)
		    // *** we ONLY do this because this is the gaze activation!

			if (gIniPara.lCursorSize<0)
			{ // *** DEBUG MODE
				// *** warn debugger of drift correction
				get_new_font("Times Roman", 24, 1);
				wait_for_video_refresh();
				graphic_printf(target_foreground_color, -1, 1, SCRWIDTH/2, 24,
					"auto-dc: region %d, delta (%ld, %ld)",i,
					(long)rgn[i].cx-avgx, (long)rgn[i].cy-avgy);
				msec_delay(200);
				// *** wait and hide the message again
				wait_for_video_refresh();
				graphic_printf(target_background_color, -1, 1, SCRWIDTH/2, 24,
					"auto-dc: region %d, delta (%ld, %ld)",i,
					(long)rgn[i].cx-avgx, (long)rgn[i].cy-avgy);
			}

			// this is a command to the eyetracker! may cause false saccade!
			eyecmd_printf("drift_correction %ld %ld %ld %ld",
				(long)rgn[i].cx-avgx, (long)rgn[i].cy-avgy,
                (long)rgn[i].cx, (long)rgn[i].cy);
			eyemsg_printf("TRIGGER MAIN %d %ld %ld %ld %ld %ld", 
				i, avgx, avgy, rgn[i].cx, rgn[i].cy, rgn[i].dwell);

			i = current_region = -1;
			reset_regions();

			/* *** swap stimuli here - NEW function!    *** */
			present_display(1,gbm);	 /* *** show image  *** */
		    /* *** 1 means log SYNCTIME and update time *** */
		}
		last_region = i;
		return stimulus_on;
	} // EOF stimulus not yet on in gaze activation mode

	/* *** *** the rest is only reached if stimulus_on is 1 *** *** */

//	eyemsg_printf("FIXUPDATE %d %ld %ld %ld %ld",i, x, y, dur, (i>0) ? rgn[i].dwell : 0);

	if(i == current_region)	   // STILL IN CURRENT REGION 
    {
		j = (myTrialGlobal->bHasImage == FALSE) ? i : 0;
		// *** 11.6.2001 - GRAPHIC trials never use rgn other than 0
		rgn[j].dwell += dur;	      // accumulate time, position
		rgn[j].avgxsum += dur * x;
		rgn[j].avgysum += dur * y;
		// *** Warning: Single fixupdates not matching current_region will
		// *** be discarded, only two fixations in a row will change region
    }
  else if(i == last_region)	// TWO UPDATES IN A ROW IN NEW REGION
							// OR OUTSIDE ANY REGION
	{
		if (current_region != -1)	// only visible words are interesting
			{
				playtime = (myTrialGlobal->bHasSound == TRUE) ?
					eaSoundWhere() : -1;
					// log how far sound playing is now
				i = current_region;		// log stats of region we were in
				// compute avg. gaze position
				j = (myTrialGlobal->bHasImage == FALSE) ? i : 0;
				// *** 11.6.2001 - GRAPHIC trials never use rgn other than 0
				avgx = rgn[j].avgxsum / rgn[j].dwell;
				avgy = rgn[j].avgysum / rgn[j].dwell;
				eyemsg_printf("LEAVE WORD %d %ld %ld %ld %ld %ld %ld",
					i, avgx, avgy, x, y, rgn[j].dwell, playtime);
			}

		i = last_region;			// rescue new region to i before reset
		reset_regions();              // clear and initialize accumulators
		if (i != -1)				// log new word if any
		{
			playtime = (myTrialGlobal->bHasSound == TRUE) ?
				eaSoundWhere() : -1;
				// log how far sound playing is now
			j = (myTrialGlobal->bHasImage == FALSE) ? i : 0;
			// *** 11.6.2001 - GRAPHIC trials never use rgn other than 0
			rgn[j].dwell = dur;	      
			rgn[j].avgxsum = dur * x;
			rgn[j].avgysum = dur * y;
			/* *** log that the gaze went to a new word! *** */
			eyemsg_printf("ENTER WORD %d %ld %ld %ld %ld %s", 
				i, x, y, rgn[j].dwell, playtime,
				(myTrialGlobal->bHasImage == FALSE) ?
				rgn[i].rword : "(graphical)"
			); // *** this is the only place where rword is logged
		}
		current_region = i;			// now working with new region
	}

  if ((gIniPara.lCursorSize<0) && (i != last_region))
	{ // *** DEBUG MODE: show region numbers if changed
		playtime = (myTrialGlobal->bHasSound == TRUE) ?
			eaSoundWhere() : -1;
			// log how far sound playing is now
		get_new_font("Times Roman", 24, 1);
		wait_for_video_refresh();
		// hide previous message
		graphic_printf(target_background_color, -1, 0,
			0, 24, "%i %ld", ishown, ptshown);
		// show new message
		graphic_printf(RGB(255,0,0), -1 /* bg, -1 is invisible */,
			0 /* center */, 0 /* x */, 24 /* y */, "%i %ld", i, playtime);
		ishown = i;
		ptshown = playtime;
	}

  last_region = i;

  return stimulus_on;
}

/********* RUN A GAZE-CONTROL TRIAL  *******/

		// End recording: adds 100 msec of data to catch final events
        // Reset configuration changes made previously
static void end_trial(void)
{
  clear_full_screen_window(target_background_color);    // hide display 
  msec_delay(100);       // record additional 100 msec of data 
  stop_recording();		 // Reset link data, disable fixation event data

  if (myTrialGlobal->bHasSound) // *** new 11.6.2001
  {
	  eaSoundUnloadFile();	// free resources
	  eaSoundClose();		// free sound device
  }

  eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXATION");
  eyecmd_printf("fixation_update_interval = 0");
  eyecmd_printf("fixation_update_accumulate = 0");
}

	/* *** 3/2001: heavy changes by Eric Auer - not THAT well documented *** */

	/* Run a gaze-control trial, recording to EDF file and sending FIXUPDATE events through link */
	/* This example draws to a bitmap, then copies it to display for fast stimulus onset */ 

	// The order of operations is:
	// - Set trial title, ID for analysis
    // - Draw to bitmap and create EyeLink display graphics
	// - Create regions
	// - Drift correction
	// - start recording
	// - start recording
	// - loop till button press, timeout, or abort, and process FIXUPDATE events	
	// - stop recording, handle abort and exit
    // This trial draws its own bitmap and creates control regions	 
	// *** new name, new functionality!
	// *** gbm is bitmap with stimulus, myTrial has setup
	// *** from setup, we use lTimeLimit and bGazeActivate
int reading_trial(HBITMAP gbm, PTRIALPARAMS myTrial) 
{
	// *** trial_start is global and set up by present_display(1,...)
  UINT32 time_limit;	// *** from myTrial here...
  int button;		// the button pressed (0 if timeout) 
  int error;            // trial result code
  int key;              // key pressed

  ALLF_DATA evt;        // buffer to hold sample and event data
  int eye_used = -1;    // indicates which eye's data to display
  int stimulus_on = -1;	// indicates if stimulus is already visible
  int i,j;		// *** added j

	last_region = -1;
	current_region = -1;

	if (myTrial->bHasSound) // 11.6.2001
	{
		// sound could be left open between trials...
		if (eaSoundOpen())
		{
		  MessageBox(full_screen_window, TEXT("Could not open sound!"),
                 TEXT("Error"), MB_OK | MB_ICONERROR);
			return -42;
		}
		if (eaSoundLoadFile(myTrial->pszWavFilename))
		{
		  MessageBox(full_screen_window, TEXT("Sound file not found."),
                 TEXT("Error"), MB_OK | MB_ICONERROR);
		}
	}

	if (myTrial->bHasImage) // 11.6.2001
	{
		myTrial->bGazeActivate = FALSE;
		// *** because w/o any rgn entries,
		// *** we will wait forever otherwise!!!
		if (gIniPara.lCursorSize<0)
		{ // *** DEBUG MODE: reveal areas
			for (i=0; i<DEBUG_FLASHES; i++)
			{
				wait_for_video_refresh();	// synchronize before drawing so all seen at once
				display_bitmap(full_screen_window,
					gbm, 0, 0);  // show BITMAP
//				msec_delay(DEBUG_AREAS_DELAY/20);
				wait_for_video_refresh();
				display_bitmap(full_screen_window,
					myTrial->hTplBmp, 0, 0);  // show BITMAP
				wait_for_video_refresh();  	 // synchronize to refresh
//				msec_delay(DEBUG_AREAS_DELAY/20);
			} // EOF loop
		} // EOF revealing debug mode
	} // EOF graphic stuff here

	myTrialGlobal = myTrial;	// 11.6.2001

  time_limit = myTrial->lTimeLimit;	// *** has to be there later

  if (myTrial->bHasImage)	// *** 11.6.2001: support direct image input
  {
	  used_regions = 0;
  } else
  {
	  used_regions = rgn[NREGIONS-1].dwell;
  }
  
  if ((used_regions<0) || (used_regions>NREGIONS))
  { used_regions = NREGIONS;
  }		// *** speed up things based on evil hack in w32_text_bitmap
		// *** but at least we check for sanity...

  if (gIniPara.lCursorSize<0)
  { // *** DEBUG MODE
    //  i = present_display(0,gbm); // *** reveal everything to the debugger :-)
	  for (i=0; i<used_regions; i++)
		{
			invert_rect(&rgn[i].rdraw); // *** reveal regions to the debugger
			invert_rect(&rgn[i].rsense); // *** reveal regions to the debugger
		}
  }

	   // This supplies the title at the bottom of the eyetracker display 
// *** commenting this out - reading_trials.h has already done it
// ***  eyecmd_printf("record_status_message 'GAZE CONTROL TRIAL' ");

	   // Always send a TRIALID message before starting to record. 
	   // It marks the start of the trial and should precede any other messages
	   // It should contain trial condition data required for analysis.
// *** same here!
// *** eyemsg_printf("TRIALID GAZECTRL");

  	// Before recording, we place reference graphics on the EyeLink display
  set_offline_mode();		 // Must be offline to draw to EyeLink screen
// *** strange but true: this is done in reading_trials, too!
// ***  gbm = draw_grid_to_bitmap();   // Draw bitmap and EyeLink reference graphics

	   // DO PRE-TRIAL DRIFT CORRECTION     *** IF NOT GAZE ACTIVATED
	   // We repeat if ESC key pressed to do setup. 
  while ((myTrial->bGazeActivate == FALSE)
	  && (myTrial->bDriftCorrect == TRUE))
	  // *** Gaze Activation replaces this!
	  // *** 11.6.2001: can be switched off, too
    {              // Check link often so we can exit if tracker stopped
      if (!eyelink_is_connected()) return ABORT_EXPT;
	   // We let do_drift_correct() draw target in this example
	   // 3rd argument would be 0 if we already drew the display
	  error = do_drift_correct((INT16)(SCRWIDTH/2), (INT16)(SCRHEIGHT/2), 1, 1);
           // repeat if ESC was pressed to access Setup menu 
      if (error!=27) break;
        // The screen was erased after drift correction because we had it
        // draw the target. If we drew the target, we could erase it now.
	  // *** do we really need this loop? Was if (1) before... ***
    }

  if ((myTrial->bGazeActivate == FALSE)
	  && (myTrial->bDriftCorrect == FALSE))
  {
		// *** 11.6.2001: simply wait if neither GA nor DC
		msec_delay(gIniPara.lNoDCDelay); // *** 2.7.2001
  }

        // Configure EyeLink to send fixation updates every 4 (50) msec
		// *** and to save fixupdates if wanted ***
	if (myTrial->bLogStream == TRUE)
	{
		eyecmd_printf("file_event_filter = LEFT,RIGHT,FIXUPDATE,FIXATION,BLINK,BUTTONS,INPUT,MESSAGE");
		// *** all
	}
	else
	{
		eyecmd_printf("file_event_filter = LEFT,RIGHT,FIXATION,BLINK,BUTTONS,INPUT,MESSAGE");
		// *** all but fixupdate
		//		eyecmd_printf("file_event_filter = LEFT,RIGHT,FIXATION,MESSAGE");
		// *** FIXATION left on because it seems to stay off once switched off! :-(
	}

  eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXUPDATE");
  eyecmd_printf("fixation_update_interval = %d",FIXUPDATE_MSEC);
  eyecmd_printf("fixation_update_accumulate = %d",FIXUPDATE_MSEC);
 
// *** this is done in text_bitmap, called from reading_trials, before
// *** text_bitmap writes to rgn[] directly for that (how un-stylish!)
// *** *** init_regions();	// Initialize regions for this display
  
	// Start data recording to EDF file, BEFORE DISPLAYING STIMULUS 
	// You should always start recording 50-100 msec before required
	// otherwise you may lose a few msec of data 
// *** w32_exptsppt.h says: file{samples,events}, link{samples,events}
// *** added user control over EDF stuff, kind of


  error = start_recording( /* uses INT16 values */
	  /* *** DONT ASK ME WHY BUT SAMPLES HAVE TO BE ON FOR EDFVIEWX *** */
			(INT16)((myTrial->bLogStream == TRUE) ? 1 : 0), /* BOOL is int */
			// *** controls writing samples to file 2.7.2001
			1,	/* *** always save fixations and messages to EDF *** */
			(INT16)((myTrial->bLogStream == TRUE) ? 1 : 0), /* BOOL is int */
			/* send samples through link, too, if on 2.7.2001 */
			1);	/* send events (like gazes) through link, not samples */
  

  if(error != 0)           // ERROR: couldn't start recording
    {
      DeleteObject(gbm);   // Be sure to delete bitmap before exiting!
      return error;        // Return the error code
    }

  for (i=0; i<used_regions; i++)	// *** Write word geometries to EDF
  {
	if ((i % 50) == 0) msec_delay(100); // avoid flooding the link
	eyemsg_printf("INFO WORD %d %ld %ld %ld %ld %s",
		i, rgn[i].rdraw.left, rgn[i].rdraw.top,
		rgn[i].rdraw.right, rgn[i].rdraw.bottom,
		rgn[i].rword);	// *** log all word geometries "on location" for later use
  }
  
  msec_delay(100);	   // record for 100 msec before displaying stimulus 

	// *** if we have gaze activation, delay this!
  if (myTrial->bGazeActivate == FALSE)
  {
     trial_start = present_display(1,gbm);	/* *** show image *** */
     					/* *** 1 means log SYNCTIME    *** */
	 stimulus_on = 1;
  }
  else
  {
    wait_for_video_refresh();
	if (gIniPara.lCursorSize<0)
	{ // *** DEBUG MODE
		// *** dont hide hints from debugger!
		clear_full_screen_window(target_background_color);    // hide display 
	}
	invert_rect(&rgn[0].rdraw);	// *** gaze target: invert the
			    				// *** region where the 1st word will be!
    wait_for_video_refresh();
    trial_start = current_time();	// record the display onset time 
						// will be updated to stimulus presentation time!
    eyemsg_printf("TARGET ON");	 // message for RT recording in analysis
	// *** new: synctime now delayed to target presentation!
	// *** eyemsg_printf("SYNCTIME");	 // message marks zero-plot time for EDFVIEW
  }
  	
     // Now get ready for trial loop
  eyelink_flush_keybuttons(0);   // reset keys and buttons from tracker 
  while(getkey()) {};		 // dump any pending local keys 


     // Trial loop: till timeout or response -- added code for processing FIXUPDATE events
  while(1) 
    {                            // First, check if recording aborted 
      if((error=check_recording())!=0) return error;  

                                 // Check if trial time limit expired
      if(current_time() > trial_start+time_limit)
        {
		  if (stimulus_on == 0)
		  {
			  eyemsg_printf("DISPLAY STILL OFF BUT TIMEOUT");
		  }
          eyemsg_printf("TIMEOUT");    // message to log the timeout 
		  end_trial();                 // local function to stop recording
		  button = 0;                  // trial result message is 0 if timeout 
		  break;                       // exit trial loop
		}

                // Check for key press, and also allows Windows to pass messages
                // getkey() returns ESC to stop trial, or TERMINATE_KEY to end experiment
      key = getkey();         // get key or termination code
      button = 0;

      if(key==TERMINATE_KEY)  // ALT-F4 or CTRL-C
		{
			end_trial();         // local function to stop recording
			return ABORT_EXPT;   // return this code to terminate experiment
		}
       else if(key==27)       // ESC key
		{
	   end_trial();         // local function to stop recording
	   return SKIP_TRIAL;   // return this code if trial terminated
        }
       else if ((myTrial->lUsedButtons)>0)	// *** scan iButtonList here!
        {
           for (j=0; j<(myTrial->lUsedButtons); j++)
             if (myTrial->iButtonList[j] == key)
               {
                 // *** using the index in iButtonList for logging --
                 // *** but we add the key code in the ANSWERBUTTON log event
                 button = j | 128;  // *** not elegant at all but should work
               }
         }

       if (button != 0)	// *** used to check if a key on the list was pressed
         {
           eyemsg_printf("ANSWERBUTTON %d KEY %d", button, key);
           end_trial();	// local function to stop recording
           break;	// exit trial loop
         }

		/* BUTTON RESPONSE TEST */
		// Check for eye-tracker buttons pressed
		// This is the preferred way to get response data or end trials	
      button = eyelink_last_button_press(NULL);
      if(button!=0)       // button number, or 0 if none pressed
	{
	   eyemsg_printf("ENDBUTTON %d", button);  // message to log the button press
	   end_trial();                            // local function to stop recording
	   break;                                  // exit trial loop
	 }
  
		// GET FIXUPDATE EVENTS, PROCESS

      i = eyelink_get_next_data(NULL);    // Check for data from link
      if(i == FIXUPDATE)	  // only process FIXUPDATE events
	{
	   if(eye_used==-1)       // If we don't yet know which eye to process, set it
	     {
	        eye_used = eyelink_eye_available();  // find which eye's data present
	        switch(eye_used)			
                  {			       // select eye, add annotation
	            case RIGHT_EYE:
	              eyemsg_printf("EYE_USED 1 RIGHT");
	              break;
	            case BINOCULAR:
	              eye_used = LEFT_EYE;
	            case LEFT_EYE:
	              eyemsg_printf("EYE_USED 0 LEFT");
	              break;
	          }
	     }
	   eyelink_get_float_data(&evt);   // get a copy of the FIXUPDATE event 
	   if(evt.fe.eye == eye_used)	   // only process if it's from the desired eye?
	     {                             // get average position and duration of the update 
	       stimulus_on =
			   process_fixupdate((int)evt.fe.gavx, (int)evt.fe.gavy,
	       			 evt.fe.entime-evt.fe.sttime,
					 gbm, stimulus_on);		// Process event
			// giving it myTrial and gbm and stimulus_on is new
	     }
	}
    }       // end of loop
	   // report response result: 0=timeout, else button number
	   // *** button number will be 128 or greater for keyboard answers
	   // *** matching the allowed answer list from the control script
  eyemsg_printf("TRIAL_RESULT %d", button);
	   // Call this at the end of the trial, to handle special conditions
  return check_record_exit();
}

