/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* Sequence block of trials, including calibration and error handling */
/* Something similar can be used for each block of an experiment      */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

/* ***************************************************** */
/* *** This was w32_text_trials.c from TextExtended2 *** */
/* *** Modified 3/2001 by Eric Auer                  *** */
/* ***************************************************** */

#include <windows.h>
#include <windowsx.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"  /* header file for this experiment */

#include "global_regions.h" /* *** added this (Eric)  *** */
#include "reading_loadscript.h" /* *** ... *** */

#include "StoringAnImage.h"
#include "LoadingATextFile.h"
#include "IniFile.h"
#include "LogTable.h"
#include "AddUnicode.h"


/*********** TRIAL SETUP AND RUN **********/

int gnTrials = 0;

         // FOR EACH TRIAL:
         // - set title, TRIALID
         // - Create bitmaps and EyeLink display graphics
         // - Check for errors in creating bitmaps
         // - Run the trial recording loop
         // - Delete bitmaps
         // - Return any error code

              // Given trial number, execute trials
              // Returns trial result code

// and store Bitmap to File
int do_text_trial(int num)
{
  HBITMAP bitmap;
  RECT margins;  // margins for text
  int i;
  TCHAR pszFilename[256];
//  TCHAR pszNum[12];
  TCHAR* pszText;
  PTRIALPARAMS myTrial;

  BOOL bRet; // ***
  int trialPages = 1; // ***

  myTrial = GetTrial(num);	// *** a structure now!
  if (myTrial == NULL) return ABORT_EXPT;

//  itoa(num, pszNum, 10);
//  lstrcpy(pszFilename, TEXT("trial")); // ***
//  lstrcat(pszFilename, pszNum);
//  lstrcat(pszFilename, TEXT("page1.bmp")); // *** only 1pg/trial for now
  lstrcpy(pszFilename, myTrial->pszTrialIdent); // *** only 1pg/trial for now
  lstrcat(pszFilename, TEXT("-1.bmp"));
   
  margins.top = SCRHEIGHT/20;
  margins.left = SCRWIDTH/20;
  margins.right = SCRWIDTH - SCRWIDTH/20;
  margins.bottom = SCRHEIGHT - SCRHEIGHT/20;

         // This supplies the title at the bottom of the eyetracker display 
  eyecmd_printf("record_status_message 'READING TRIAL %d/%d PAGE 1/1 IDENT %s' ",
                num, gnTrials, myTrial->pszTrialIdent); // ***

        // Always send a TRIALID message before starting to record. 
        // It should contain trial condition data required for analysis.
  eyemsg_printf("TRIALID %sPAGE1", myTrial->pszTrialIdent);
  	// *** limited length??? -> use %d num, not ident

  get_new_font(gIniPara.pszFontName, gIniPara.lFontHeight, 0);
  // get_new_font("Times Roman", SCRHEIGHT/25, 0);
	
  // *** yeah, I know, file access during experiment is not optimal
  // *** NEW May 21 2001: text may be stored in script to avoid loading from other files
  // *** new 11.6.2001 - never try to load an image as text ;-)
  if ((myTrial->bHasImage == FALSE)
	  && (myTrial->bTextInline == FALSE))
	{
	  bRet = LoadText(myTrial->pszTextFilename, &trialPages); // ***
	  if (!bRet)
		{
	      eyemsg_printf("TRIAL ERROR");
		  MessageBox(full_screen_window, TEXT("Text file not found."),
                 TEXT("Error"), MB_OK | MB_ICONERROR);
		  if (gIniPara.lCursorSize<0)
			{ // *** DEBUG MODE
		  // *** tell which file is missing
			  get_new_font("Times Roman", 24, 1);
			  graphic_printf(target_foreground_color, -1, 1, SCRWIDTH/2, 24,
				"Missing: Trial %d [%s]", num, myTrial->pszTextFilename);
			}
			return SKIP_TRIAL;
		}	// EOF load error
	} // EOF load TEXT from file

  // ***
  // *** if trialPages > 1, the text file contains not yet usable additional data!
  // ***
 
	// *** new 11.6.2001: support for graphical trials
    if (myTrial->bHasImage) // then *GRAPHICAL* trial
	{
		// must load the bitmap and the template
		// to bitmap and myTrial->hTplBmp (see w32_demo.h)
		bitmap = bmp_file_bitmap(myTrial->pszTextFilename,0,0,READING_TEXT_BACK);
		if (bitmap == NULL)
		{
		  MessageBox(full_screen_window, TEXT("Image file not found."),
                 TEXT("Error"), MB_OK | MB_ICONERROR);
		}
		myTrial->hTplBmp = bmp_file_bitmap(myTrial->pszTplFilename,0,0,READING_TEXT_BACK);
		if (myTrial->hTplBmp == NULL)
		{
		  MessageBox(full_screen_window, TEXT("Template image file not found."),
                 TEXT("Error"), MB_OK | MB_ICONERROR);
		}

		i = reading_trial(bitmap, myTrial);
		// myTrial passes along lots of other stuff, e.g.
		// data needed for sound playing
		DeleteObject(myTrial->hTplBmp); // free memory
		DeleteObject(bitmap); // free memory
	} // EOF *GRAPHICAL* trial
	else // *TEXT* trial
	{
	  // *** NEW May 21 2001: this may use either file contents loaded by LoadText
	  // ***                  or inlined text from the script
		if (myTrial->bTextInline)
		{
			pszText = myTrial->pszText;	// text is inlined 
		}
		else
		{
			pszText = GetPage(1);			// text was read from another file by LoadText
		}

		if (pszText) // if loading worked
		{
			// *** change text_bitmap to write to rgn[] !!!
			// *** was RGB(0,0,0), RGB(255,255,255)
			bitmap = text_bitmap(pszText, READING_TEXT_COLOR, READING_TEXT_BACK,
				margins, gIniPara.lLineSpacing, 1);

			if (gIniPara.lSaveTextBitmaps != 0)
			{
				StoreBitmapToFile(full_screen_window, bitmap, pszFilename);
			}; // *** 2.7.2001 - storing now based on global ini file parameter

			i = reading_trial(bitmap, myTrial); // *** not only time limit passed
			// *** used info: bGazeActivate, bLogStream, lTimeLimit,
			// *** lUsedButtons, int iButtonList[] and others, see
			// *** reading_loadscript.h - new 11.6.2001: sound playing info
			DeleteObject(bitmap); // free memory
		}

		if (!myTrial->bTextInline) ReleaseText(); // release file read buffers if any

	} // EOF  *TEXT* TRIAL

  return i;
}


/*********** TRIAL LOOP **************/

	/* This code sequences trials within a block */
	/* It calls run_trial() to execute a trial, */
	/* then interperts result code. */
	/* It places a result message in the EDF file */
	/* This example allows trials to be repeated */
	/* from the tracker ABORT menu. */
int run_trials(LPSTR lpCmdLine) // *** new 11.6.2001 - get command line
{
	int i;
	int trial;
	BOOL bRet;
	TCHAR /* LPCSTR */ errmsg[512];

	target_foreground_color = READING_TARGET_COLOR; // set background for calibration
	target_background_color = READING_TARGET_BACK;  // This should match the display 
	set_calibration_colors(target_foreground_color, target_background_color); 
    
	// get values from ini file

	if (!ReadParams(&gIniPara))
	{
		MessageBox(full_screen_window, TEXT("No ini file, using defaults."),
			TEXT("Warning"),
			MB_OK | MB_ICONWARNING);
		GetDefaultValues(&gIniPara);
		WriteParams(&gIniPara);
	}

	// load script file - command line overrides file name from ini file
	// (new 11.6.2001)

	if ((lpCmdLine != NULL) && (*lpCmdLine != '\0'))
	{ // if we have something on the command line...
		MessageBox(full_screen_window,
			TEXT("Command line overrides script file name given in ini file"),
			TEXT("Please note"),
			MB_OK | MB_ICONEXCLAMATION);
		// strcpy(gIniPara.pszTextFilename, lpCmdLine);
		// as we are copying LPSTR (char *) to TCHAR *,
		// better be on the safe side.
		for (i=0;i<200;i++)
		{
			if ((gIniPara.pszTextFilename[i] = lpCmdLine[i]) == '\0')
				break;
		}
		gIniPara.pszTextFilename[200] = '\0';
	}

	bRet = LoadScript(gIniPara.pszTextFilename, &gnTrials);
	if (!bRet)
	{
		eyemsg_printf("TRIAL ERROR");
		lstrcpy(errmsg,"Script file not found: ");
		lstrcat(errmsg,gIniPara.pszTextFilename);
		MessageBox(full_screen_window, errmsg, TEXT("Error"),
			MB_OK | MB_ICONERROR);
		return 0;
	}

 	/* PERFORM CAMERA SETUP, CALIBRATION */
	if (gIniPara.lCursorSize<0)
	{ // *** DEBUG MODE
		MessageBox(full_screen_window, TEXT("Starting tracker setup now."),
			TEXT("Notice"),
			MB_OK | MB_ICONINFORMATION);
	}
	do_tracker_setup();

	/* loop through trials */
	for(trial=1;trial<gnTrials;trial++) // *** 1based index!
										// *** but still using <, not <= ...
	{
		if (gIniPara.lCursorSize<0)
		{ // *** DEBUG MODE
			// *** show when next iteration of trial loop begins ***
			MessageBox(full_screen_window, TEXT("Prepare for next trial."),
				TEXT("Notice"),
				MB_OK | MB_ICONINFORMATION); // ERROR WARNING INFORMATION EXCLAMATION ... 0
		}
		if(eyelink_is_connected()==0 || break_pressed())    /* drop out if link closed */
		{
			return ABORT_EXPT;
		}
				/* RUN THE TRIAL */
		LogTableSetPage(trial);
		i = do_text_trial(trial); // *** see above - calls reading_trial

		switch(i)         	/* REPORT ANY ERRORS */
		{
			case ABORT_EXPT:        /* handle experiment abort or disconnect */
				eyemsg_printf("EXPERIMENT ABORTED");
				ReleaseScript();
				return ABORT_EXPT;
			case REPEAT_TRIAL:	  /* trial restart requested */
				eyemsg_printf("TRIAL REPEATED");
				trial--;
				break;
			case SKIP_TRIAL:	  /* skip trial */
				eyemsg_printf("TRIAL ABORTED");
				break;
			case TRIAL_OK:          // successful trial
				eyemsg_printf("TRIAL OK");
				break;
			default:                // other error code
				eyemsg_printf("TRIAL ERROR");
				break;
		}
	}  // END OF TRIAL LOOP

	ReleaseScript();
	return 0;
}

