/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* Draw left-justified text in margins, to display or bitmap */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

/* *** Changed 3/2001 by Eric Auer to write to rgn[] and added \_  *** */
/* *** \_ is an invisible non breakable space, rgn[] has geom info *** */

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"

#include "LogTable.h"
#include "AddUnicode.h"
#include "global_regions.h" // *** added this...
#include "IniFile.h" // *** 2.7.2001 to control padding

/********** DRAW FORMATTED TEXT ***********/

       // Draw text within margins, left-justified, with word-wrap.
       // Draw to <hdc> context, so it supports display and bitmaps   
       // (bitmap must already be created and selected into context).
       // Draw boxes for each word on EyeLink tracker display if <dotrack> set
       // Use font selected with get_new_font(), and draws it in <color>
       // RECT <margins> sets margins, and <lspace> sets pixels between lines
void draw_text_box(HDC hdc, TCHAR *txt, COLORREF color, RECT margins, int lspace, int dotrack)
{
  LOGTABLEPARA para;
  short x, y;				// drawing position
  TCHAR *sword, *eword;     // word start, end pointer
  TCHAR *sCLUSTER;			// word cluster start pointer *** new May 21 2001
  int swx, ewx;				// word start, end position
  TCHAR *sispace, *c;       // text pointers
  int height,pad;           // height of characters, *** padding to increase box height
  SIZE size;
  TEXTMETRIC tm;
  int i; 		// *** index for rgn[]
  int used_regions,wrapped; // *** wrapped is new May 21 2001

  if(dotrack) eyecmd_printf("set_idle_mode");   // must be in idle mode to draw graphics
  if(dotrack) eyecmd_printf("clear_screen 0");	// clear eye tracker display

  SetBkMode(hdc, TRANSPARENT);        // set text color
  SetTextColor(hdc, color);
  SelectObject(hdc, current_font);    // Select cached font into context

  GetTextMetrics(hdc, &tm);           // Get height of characters
  height = tm.tmHeight;

  // *** 2.7.2001 padding now controlled by .ini file
  if (gIniPara.lTextPaddingSize<0)
  {
	  pad = (lspace - (height+(-gIniPara.lTextPaddingSize)))/2;
	  // *** pad to leave only N ypixels between boxes
  } else
  {
	  pad = gIniPara.lTextPaddingSize;
	  // *** pad N pixels above and N pixels below boxes
  }

  if (pad < 0) pad = 0;				// *** pad is new May 21 2001
  if (pad > (lspace/2)) pad = lspace/2; // *** avoid overlapping

  SetTextJustification(hdc, 0, 0) ;   // set normal text spacing
  SetTextAlign(hdc, TA_LEFT | TA_TOP);

  x = (short)margins.left;          // initial drawing position
  y = (short)margins.top;  
  c = txt;                          // initial text to scan
  para.nLine = 1;
  para.nWordNo = 1;
  // *** make sure to read the global_regions.h about rgn[], rword is currently char[], not TCHAR[]
  used_regions = 0;					// *** returned for reading_trials

  
  while(*c)   // loop through lines of text -- parsing loop #######
    {

	  if ((*c!='\\') || (*(c+1)!='_'))	// *** only start new cluster if no \_
		{
nextCLUSTER:			// *** START NEW CLUSTER: "test\_ing" is 1 cluster but 2 "words"
	  sCLUSTER = c;
	  wrapped = used_regions;	// *** yes, will HANG if cluster does not fit to line...
		}
nextword:                 // START NEW WORD:
      sispace = c;           
      while(isspace(*c)) c++;	// scan over leading spaces if any
      if(c!=sispace)          // advance over the spaces (if any)
        {
		  // computes the width and height of the space
          GetTextExtentPoint32(hdc, sispace, c-sispace, &size);                    
          x += (short)size.cx;
        }

      if(*c==0) break;    // handle end-of-text 

	// *** this is the standard "\\n" user-newline feature provided by w32_text_bitmap:
      if((*c == '\\') && (*(c+1) == 'n'))        // handle end-of-line *before* word
        {
          c+=2;            // skip over end-of-line characters

nextline:                        // START NEW LINE:
          x = (short)margins.left;                      // return to start of line
          y += lspace;                           // top of new  line
		  para.nLine++;                          // increments line counter
		  para.nWordNo = 1;                      // word number begins with 1

          if(y+height > margins.bottom)
		  {
			MessageBox(full_screen_window, TEXT("draw_text_box: too much text."),
				TEXT("Warning"), MB_OK | MB_ICONERROR); // ERROR WARNING
			break;  // stop if it would go outside margins
		  }

          goto nextCLUSTER;                         // continue to print (was: nextword)
		
		} // EOF if (\\n or jumped to nextline)
		else	/* *** have to handle ALL backslash sequences or will loop endless! *** */
		{
			if ((*c == '\\') && (*(c+1) != 'n'))
			{
				c+=2;	// *** skip over any other \\? sequences *in front* of a word
			}
		}

	  swx = x;            // continue in current line: 

      sword = c;                                  // record start of this word
	  sscanf(c, TEXT("%s"), para.pszWord);              // reads the word
      while(*c && 
		  !isspace(*c) && 
		  *c!='\n' && *c!='\r' && *c!='\\' /* *** backslash always ends a word *** */
		  ) c++;  // scan till end of word

      if(sword==c) goto nextword;                 // no word: continue by skipping spaces (!?)

	  eword = c;                                  // record end of word
	  GetTextExtentPoint32(hdc, sword, eword-sword, &size);   // width of word                 
	  ewx = size.cx + swx;                        // position of end of word

      // *** write to rgn[] -- always on for now!
      i = used_regions;
      if (used_regions<(NREGIONS-1)) used_regions++; // simple clipping to array

      SetRect(&(rgn[i].rdraw), swx, y, ewx, y+height);
	  rgn[used_regions-1].rwordStrlen = eword-sword;

	  for (i=0; i<(eword-sword); i++) rgn[used_regions-1].rword[i] = sword[i];
          // *** this may involve unicode -> text conversion if TCHAR is used
      rgn[used_regions-1].rword[eword-sword] = '\0';
	  // *** EOF write to rgn[] - was "if dotrack" in earlier versions

      if ((ewx > margins.right) &&
		  ((*c!='\\') || (*(c+1)!='_')))	// would it extend past right margin? *** protect \_
        {
          c = sCLUSTER;					// restore text pointer *** (was sword)
          used_regions = wrapped;		// *** start over with this cluster (not only word)
          goto nextline;				// start a new line with current word *** cluster
        }

      x = ewx;                                     // update current printing location

  } // EOF parsing loop ########


  for (i=0; i<used_regions; i++)	// *** changed May 21 2001: process rgn[] AFTER parsing
  {
	  if(dotrack)
	  {		// *** draw box on EyeLink display
		para.rectWord = rgn[i].rdraw; // members are left top right bottom
		eyecmd_printf("draw_box %d %d %d %d 14",
			  para.rectWord.left,  para.rectWord.top, 
			  para.rectWord.right, para.rectWord.bottom); 
	    LogTable(&para);
		para.nWordNo++;
	  }

	  // *** initialize rest of members
	  rgn[i].triggered = 0;
      rgn[i].dwell = 0;
      rgn[i].avgxsum = 0;
      rgn[i].avgysum = 0;
      rgn[i].cx = (rgn[i].rdraw.left + rgn[i].rdraw.right) / 2;
      rgn[i].cy = (rgn[i].rdraw.top + rgn[i].rdraw.bottom) / 2;
	  // *** changed May 21 2001: OLD: rsense is smaller than rdraw (border 3..4)
	  // ***                      NEW: rsense is at least as large as rdraw (plus y pad)
	  SetRect(&(rgn[i].rsense), rgn[i].rdraw.left, rgn[i].rdraw.top-pad,
	  rgn[i].rdraw.right, rgn[i].rdraw.bottom+pad);

	  // *** draw words to bitmap
	  TextOut(hdc, rgn[i].rdraw.left, rgn[i].rdraw.top, rgn[i].rword, rgn[i].rwordStrlen);
  
  } // EOF box and bitmap printing loop

  
  SelectObject(hdc, GetStockObject(SYSTEM_FONT));  // deselect the font

  rgn[NREGIONS-1].dwell = used_regions; // *** evil hack to speed up things

}


/************ CREATE BITMAP WITH FORMATTED TEXT ***********/

      // Draw text to full-screen sized bitmap, cleared to <bgcolor>
      // Draw boxes for each word on EyeLink tracker display if <dotrack> set
      // Use font selected with get_new_font(), and draws it in <fgcolor>
      // RECT <margins> sets margins, and <lspace> sets pixels between lines
HBITMAP text_bitmap(TCHAR *txt, COLORREF fgcolor, COLORREF bgcolor, 
                               RECT margins, int lspace, int dotrack)
{
  HDC hdc;
  HBITMAP hbm;
  HDC mdc;
  HBRUSH oBrush;
  HBITMAP obm;

  hdc = GetDC(NULL);                 
  mdc = CreateCompatibleDC(hdc);     // create display-compatible memory context
  hbm = CreateCompatibleBitmap(hdc, SCRWIDTH, SCRHEIGHT);
  obm = SelectObject(mdc, hbm);      // create DDB bitmap, select into context
                                     
  oBrush = SelectObject(hdc, CreateSolidBrush(bgcolor | 0x02000000L));  // brush to fill with 
  PatBlt(mdc, 0, 0, SCRWIDTH, SCRHEIGHT, PATCOPY);                      // CLEAR BITMAP
  DeleteObject(SelectObject(hdc, oBrush));

  draw_text_box(mdc, txt, fgcolor, margins, lspace, dotrack);  // DRAW THE TEXT

  SelectBitmap(mdc, obm);     // Release the GDI resources
  DeleteDC(mdc);
  ReleaseDC(NULL, hdc);
  return hbm;                 // Return the new bitmap
}

